#ifdef SCCS
static char sccsid[]="@(#)slider.c	1.1 Stellar 89/02/09";
#endif
/*
			Copyright 1989 by
			Stellar Computer Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Stellar Computer Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
*/
/*
 * LUI Sliders
 */

#include <stdio.h>

#define LUI_SLIDER_DECLARE
#include "lui.h"

static LUI_FNCP LUI_SliderFunction;

/******************************************************************************/
 LUI_SLIDER_BOX *
LUI_SliderBoxOpen(name, parent, x, y, border, func)
    char *name;
    Window parent;
    int x, y, border;
    LUI_FNCP func;
{
    LUI_SLIDER_BOX *slider_box;
    Window LUI_SliderBoxCreateWindow();
    int LUI_SliderBoxDraw();
    
    /* Check for errors */
    if (LUI_SliderBox) {
	fprintf(stderr, "LUI_SliderBoxOpen: SliderBox %s still open!\n", 
		LUI_SliderBox->name);
	return NULL;
    }
    
    /* Initialize placement variables */
    LUI_SliderBoxWidth  = 0;
    LUI_SliderBoxHeight = 0;

    /* Create the slider box window */
    LUI_SliderBoxWindow = LUI_CreateWindow(parent, 0, 0);
    
    /* Allocate the slider box */
    if (!(slider_box = (LUI_SLIDER_BOX *)malloc(sizeof(LUI_SLIDER_BOX)))) {
	fprintf(stderr, "LUI_SliderBoxOpen: can't allocate slider box\n");
	return NULL;
    }
    
    /* Set Current slider box and function */
    LUI_SliderBox      = slider_box;
    LUI_SliderFunction = func;
    
    /* Setup slider box variables */
    strncpy(slider_box->name, name, sizeof(slider_box->name));
    slider_box->x           = x;
    slider_box->y           = y;
    slider_box->window      = LUI_SliderBoxWindow;
    slider_box->border_size = border;

    /* Dimension the slider box label */
    LUI_SliderBox->label_x = 0;
    LUI_SliderBox->label_y = 0;
    
    /* Add slider box to head of list */
    slider_box->np = LUI_SliderBoxHead;
    LUI_SliderBoxHead  = slider_box;
    
    /* Add Expose event */
    LUI_EventAdd(LUI_SliderBoxWindow, ExposureMask, LUI_SliderBoxDraw);
    
    return(slider_box);
}

/******************************************************************************/
LUI_SliderBoxClose(name)
    char *name;
{
    XSizeHints size_hints;
    int x, y, width, height, bs;
    
    /* Check for errors */
    if (!LUI_SliderBox) {
	fprintf(stderr, "LUI_SliderBoxClose: Can't find open LUI_SliderBox!\n");
	return 0;
    }

    /* Set width and height */
    bs = LUI_SliderBox->border_size;
    LUI_SliderBox->width  = LUI_SliderBoxWidth  + bs;
    LUI_SliderBox->height = LUI_SliderBoxHeight + bs;

    /* Reset size of window and reposition it */
    x      = LUI_SliderBox->x;
    y      = LUI_SliderBox->y;
    width  = LUI_SliderBox->width;
    height = LUI_SliderBox->height;
    XResizeWindow(LUI_Display, LUI_SliderBoxWindow, width, height);
    XMoveWindow(LUI_Display, LUI_SliderBoxWindow, x, y);

    /* Reset Slider Globals */
    LUI_SliderBox = NULL;
    return 1;
}

/******************************************************************************/
LUI_SliderBoxVisible(name, on_off)
    char *name;
    int on_off;
{
    register LUI_SLIDER_BOX *sp;

    for (sp = LUI_SliderBoxHead; sp; sp = sp->np) 
      if (!strcmp(sp->name, name)) break;

    if (sp) {
	if (on_off) {
	  XMapWindow(LUI_Display, sp->window);
	  sp->visible = 1;
        }
	else {
	  XUnmapWindow(LUI_Display, sp->window);
	  sp->visible = 0;
        }
    }
    else 
      fprintf(stderr, "LUI_SliderBoxVisible: can't find SliderBox %s\n", name);
}

/******************************************************************************/
LUI_SliderBoxMove(name, x, y)
    char *name;
    int x, y;
{
    register LUI_SLIDER_BOX *sp;
    
    for (sp = LUI_SliderBoxHead; sp; sp = sp->np)
      if (!strcmp(sp->name, name)) break;

    if (sp) 
	XMoveWindow(LUI_Display, sp->window, x, y);
}

/******************************************************************************/
LUI_SliderBoxQuery(name, x, y, width, height)
    char *name;
    int *x, *y, *width, *height;
{
    register LUI_SLIDER_BOX *sp;

    for (sp = LUI_SliderBoxHead; sp; sp = sp->np) 
      if (!strcmp(sp->name, name)) break;

    if (sp) {
	*x = sp->x;
	*y = sp->y;
	*width = sp->width;
	*height = sp->height;
    }
    else 
	fprintf(stderr, "LUI_SliderBoxQuery: can't find SliderBox %s\n", name);
}

/******************************************************************************/
 LUI_SLIDER_BOX *
LUI_SliderBoxLookupWindow(window)
    Window window;
{
    register LUI_SLIDER_BOX   *sp;

    for (sp = LUI_SliderBoxHead; sp; sp = sp->np) 
	if (sp->window == window) return(sp);

    return(NULL);
}

/******************************************************************************/
 LUI_SLIDER_BOX *
LUI_SliderBoxLookupName(name)
    char *name;
{
    register LUI_SLIDER_BOX *sp;

    for (sp = LUI_SliderBoxHead; sp; sp = sp->np) 
	if (!(strcmp(name, sp->name))) return(sp);

    return(NULL);
}
   
/******************************************************************************/
LUI_SliderBoxDraw(window, event)
    Window window;
    XEvent *event;
{
    LUI_SLIDER_BOX *sp;

    /* Squelch multiple redraws */
    if (event->xexpose.count > 0)
      return;

    if ((sp = LUI_SliderBoxLookupWindow(event->xexpose.window)) == NULL)
      return;

    XFillRectangle(LUI_Display, window, LUI_GC_black, 
		   0, 0, sp->width, sp->height);

    LUI_FrameWindow2(window, sp->width, sp->height, sp->border_size);
}

/******************************************************************************/
 LUI_SLIDER *
LUI_SliderCreate(label, low, high, init, type,x,y,width,height,index,func)
    char *label;
    float low, high, init; 
    int type, x, y;
    int width, height;
    int index;
    LUI_FNCP func;
{
    register LUI_SLIDER *slider;
    Window window;
    XSetWindowAttributes window_attributes;
    int xoff, yoff, LUI_SliderProcess();

    /* verify parms are OK */
    if (low>high || init<low || init>high) {
	fprintf(stderr,"LUI_SliderCreate %s Improper low=%g,high=%g,init=%g\n",
		label, low, high, init);
	return NULL;
    }
   /** Added by Marief June 21st 1989 */
    /* index of the slider in the list */
    if (index == NULL ) index = y;   

    /* Adjust position based on Slider size */
    x = (x * LUI_SLIDER_X_SIZE) + LUI_SliderBox->border_size;
    y = (y * LUI_SLIDER_Y_SIZE) + LUI_SliderBox->border_size;
    
    /* Adjust max width and height of Slider Box window */
    xoff = LUI_SLIDER_X_SIZE;
    yoff = LUI_SLIDER_Y_SIZE;
    if ((x + xoff) > LUI_SliderBoxWidth ) LUI_SliderBoxWidth  = x + xoff;
    if ((y + yoff) > LUI_SliderBoxHeight) LUI_SliderBoxHeight = y + yoff;
    
    /* Create the slider window */
    window_attributes.background_pixel = LUI_Color_FG;
    window = XCreateWindow(LUI_Display, LUI_SliderBoxWindow, 
			   x, y, 
			   LUI_SLIDER_X_SIZE, LUI_SLIDER_Y_SIZE,
			   0,
			   LUI_Depth,
			   InputOutput,
			   CopyFromParent,
			   CWBackPixel,
			   &window_attributes);

    /* Add into Event Table */
    LUI_EventAdd(window, ExposureMask | EnterWindowMask | LeaveWindowMask |
	     ButtonMotionMask | ButtonPressMask | ButtonReleaseMask, LUI_SliderProcess);

/*********Modified by Marief May 18,1989 ************/
    LUI_EventAdd( window, ButtonMotionMask | ButtonPressMask | 
		 ButtonReleaseMask, LUI_SliderFunction);

    /* Allocate a new slider */
    if (!(slider = (LUI_SLIDER *)malloc(sizeof(LUI_SLIDER)))) {
	fprintf(stderr, "LUI_SliderCreate: can't allocate new slider\n");
	return NULL;
    }

    /* Initialize slider */
    if(label==NULL)slider->label="";
    else strcpy(slider->label= (char *)malloc(strlen(label)+1), label);
    slider->state      = LUI_SLIDER_INACTIVE;
    slider->type       = type;
    slider->x          = x;
    slider->y          = y;
    slider->low	       = low;
    slider->high       = high;
    slider->value      = init;
    slider->index      = index;
    slider->func       = func;
    slider->window     = window;
    slider->slider_box = LUI_SliderBox;
    slider->hilite     = 0;

    /* Add slider to head of list */
    slider->np = LUI_SliderHead;
    LUI_SliderHead = slider;

    /* Map the slider window */
    XMapWindow(LUI_Display, window);

    return(slider);
}

/******************************************************************************/
LUI_SliderDraw(slider)
    register LUI_SLIDER *slider;
{
    XPoint pts[4];
    int  x, y, w, h;
    int  xc, yc, size;
    int  xvalue;
    char caption[128], valstr[128];

    /* Init some values */
    xc = LUI_SLIDER_X_SIZE / 2;
    yc = LUI_SLIDER_Y_SIZE / 2;
    xvalue = LUI_SLIDER_BAR_X_MARGIN + (LUI_SLIDER_BAR_LEN) *
      (slider->value-slider->low) / (slider->high - slider->low);

    /* Fill the background and draw border */
    XFillRectangle(LUI_Display, slider->window, 
		   slider->hilite ? LUI_GC_white : LUI_GC_xl_grey,
		   0, 0, LUI_SLIDER_X_SIZE, LUI_SLIDER_Y_SIZE);
    XDrawRectangle(LUI_Display, slider->window, LUI_GC_black,
		   0, 0, LUI_SLIDER_X_SIZE, LUI_SLIDER_Y_SIZE);
    XDrawRectangle(LUI_Display, slider->window, LUI_GC_black,
		   1, 1, LUI_SLIDER_X_SIZE-2, LUI_SLIDER_Y_SIZE-2);

    /* Draw slider bar */
    x = LUI_SLIDER_BAR_X_MARGIN + 6;
    y = LUI_SLIDER_BAR_Y_MARGIN;
    w = LUI_SLIDER_BAR_LEN - 12;
    h = 6;
    XFillRectangle(LUI_Display, slider->window, LUI_GC_white,
		   x-6, y, x+12, 2*h);

    XFillRectangle(LUI_Display, slider->window, LUI_GC_md_grey,
		   x+w, y, 6, 2*h);

    pts[0].x = x-6; pts[0].y = y;   pts[1].x = x+w+6; pts[1].y = y;
    pts[2].x = x+w; pts[2].y = y+h; pts[3].x = x;     pts[3].y = y+h;
    XFillPolygon(LUI_Display, slider->window, 
		 LUI_GC_lt_grey, pts, 4, Convex, CoordModeOrigin);
    XDrawLines(LUI_Display, slider->window, 
		 LUI_GC_black, pts, 4, CoordModeOrigin);

    y += h;
    pts[0].x = x;     pts[0].y = y;   pts[1].x = x+w;   pts[1].y = y;
    pts[2].x = x+w+6; pts[2].y = y+h; pts[3].x = x-6; pts[3].y = y+h;
    XFillPolygon(LUI_Display, slider->window, 
		 LUI_GC_sl_grey, pts, 4, Convex, CoordModeOrigin);
    XDrawLines(LUI_Display, slider->window, 
		 LUI_GC_black, pts, 4, CoordModeOrigin);

    XDrawRectangle(LUI_Display, slider->window, LUI_GC_black,
		   x-6, LUI_SLIDER_BAR_Y_MARGIN, w+12, 2*h);

    /* Pointer tick */
    x = xvalue;
    y = LUI_SLIDER_BAR_Y_MARGIN;
    w = 6;
    h = 6;

    pts[0].x = x;   pts[0].y = y;     pts[1].x = x+w; pts[1].y = y+h;
    pts[2].x = x;   pts[2].y = y+h*2; pts[3].x = x-w; pts[3].y = y+h;
    XFillPolygon(LUI_Display, slider->window, 
		 LUI_GC_dk_grey, pts, 4, Convex, CoordModeOrigin);

    y += 2;
    w -= 2;
    h = 4;
    pts[0].x = x;   pts[0].y = y;     pts[1].x = x+w; pts[1].y = y+h;
    pts[2].x = x;   pts[2].y = y+h*2; pts[3].x = x-w; pts[3].y = y+h;
    XFillPolygon(LUI_Display, slider->window, 
		 LUI_GC_md_grey, pts, 4, Convex, CoordModeOrigin);

    y += 2;
    w -= 2;
    h = 2;
    pts[0].x = x;   pts[0].y = y;     pts[1].x = x+w; pts[1].y = y+h;
    pts[2].x = x;   pts[2].y = y+h*2; pts[3].x = x-w; pts[3].y = y+h;
    XFillPolygon(LUI_Display, slider->window, 
		 LUI_GC_white, pts, 4, Convex, CoordModeOrigin);

    /* Add a caption */
    LUI_SliderMakeString(slider, slider->value, valstr);
    sprintf(caption, "%s = %s", slider->label, valstr);
    size = strlen(caption);
    
    xc -= XTextWidth(LUI_Font, caption, size) / 2.0 + 0.5;
    yc += LUI_Font_yoff / 2.0 + 0.5 - 8;
    
    XDrawString(LUI_Display, slider->window, LUI_GC_fore, xc, yc, caption, size);

    /* Low and high labels */
    size = LUI_SliderMakeString(slider, slider->low, valstr);
    xc = LUI_SLIDER_BAR_X_MARGIN - 3;
    XDrawString(LUI_Display, slider->window, LUI_GC_fore, xc, yc, valstr, size);

    size = LUI_SliderMakeString(slider, slider->high, valstr);
    xc = LUI_SLIDER_BAR_X_MARGIN + LUI_SLIDER_BAR_LEN - XTextWidth(LUI_Font, valstr, size) + 3;
    XDrawString(LUI_Display, slider->window, LUI_GC_fore, xc, yc, valstr, size);
}

/******************************************************************************/
 LUI_SLIDER *
LUI_SliderLookup(name)
    char *name;
{
    register LUI_SLIDER   *sp;

    for (sp = LUI_SliderHead; sp; sp = sp->np) 
	if (!(strcmp(sp->label, name))) return(sp);

    return(NULL);
}

/******************************************************************************/
 LUI_SLIDER *
LUI_SliderLookupWindow(window)
    Window window;
{
    register LUI_SLIDER *sp;

    for (sp = LUI_SliderHead; sp; sp = sp->np) 
	if (sp->window == window) return(sp);

    return(NULL);
}

/******************************************************************************/
LUI_SliderHighlight(slider, on)
    register LUI_SLIDER *slider;
    register int on;
{
    if (on) {
	LUI_Cursor(slider->window, LUI_UP_CURSOR); 
	slider->hilite = 1;
	LUI_SliderDraw(slider);
    }
/*********Modified by Marief May 18,1989 ************/
    else {
	LUI_Cursor(slider->window, LUI_MAIN_CURSOR);
	slider->hilite = 0;
	LUI_SliderDraw(slider);
    }
}

/******************************************************************************/
 static 
LUI_SliderProcess(window, event)
    Window window;
    XEvent *event;
{
    LUI_LABEL *label;
    LUI_SLIDER *slider;
    int     cursor;

    /* Get the slider function */
    if (!(slider = LUI_SliderLookupWindow(window))) {
	fprintf(stderr, "LUI_SliderProcess: can't find slider for window %d\n", window);
	return;
    }

    label = slider->slider_box->label;

    /* Process the event */
    switch (event->type) {
      case EnterNotify:
	LUI_SliderHighlight(slider, 1);
	break;
	
      case LeaveNotify:
	if (slider->state == LUI_SLIDER_INACTIVE)
	  LUI_SliderHighlight(slider, 0);
	break;

      case Expose:
	LUI_SliderDraw(slider);
	break;

      case ButtonPress:
	slider->state = LUI_SLIDER_ACTIVE;
        slider->button = event->xbutton.button; /* WLH added 12-12-91 */
	/* fall through to .... */
      case MotionNotify:
	if(slider->state == LUI_SLIDER_ACTIVE) {
	    float x_in_pixels; /* inside slide bar range */

	    if(event->type==ButtonPress)
	      x_in_pixels = event->xbutton.x - LUI_SLIDER_BAR_X_MARGIN;
	    else x_in_pixels = event->xmotion.x - LUI_SLIDER_BAR_X_MARGIN;
	    
	    /* clamp value: */
	    if(x_in_pixels<0) x_in_pixels = 0;
	    if(x_in_pixels> LUI_SLIDER_BAR_LEN)
	      x_in_pixels= LUI_SLIDER_BAR_LEN;
	    
	    /* map from slider pixels to application range: */
	    slider->value = slider->low +
	      (slider->high-slider->low)  *  
		x_in_pixels/(LUI_SLIDER_BAR_LEN);

	    if (slider->type == LUI_SLIDER_INT) 
	      slider->value = (int)(slider->value + 0.5);

	    if (slider->old_value != slider->value) {
		if (slider->func)
		  (*(slider->func))(slider);
		
		LUI_SliderDraw(slider);
	    }

	    slider->old_value = slider->value;
	}
	break;
	
      case ButtonRelease:
	slider->state = LUI_SLIDER_INACTIVE;
	slider->hilite = 0;

	/* This might cause problems: Added to allow an application to */
	/* only do stuff on a button release                           */

	if (slider->func)
	  (*(slider->func))(slider);

	LUI_SliderDraw(slider);
		
	break;
    }

    /* Flush remaining motion events in event queue */
    while (QLength(LUI_Display) > 0) {
	XEvent ne;

	XPeekEvent(LUI_Display, &ne);
	
	/* Don't flush if its a button press or release */
	if (ne.type != MotionNotify) 
	  break;

	/* Get the next event */
	XNextEvent(LUI_Display, &ne);
    }
}

/******************************************************************************/
LUI_SliderValueSet(name, val)
    char *name;
    float val;
{	
    LUI_SLIDER * slider;

    if (!(slider = LUI_SliderLookup(name))) {
	fprintf(stderr, "LUI_SliderValueSet: can't find slider %s\n", name);
	return(0);
    }
    
    slider->value = val;
    return(1);
}

/******************************************************************************/
 float
LUI_SliderValueGet(name)
    char *name;
{
    LUI_SLIDER *slider;

    if (!(slider = LUI_SliderLookup(name))) {
	fprintf(stderr, "LUI_SliderValueGet: can't find slider %s\n", name);
	return(0.0);
    }
    return(slider->value);
}

/******************************************************************************/
LUI_SliderMakeString(slider, value, retstr)
    LUI_SLIDER *slider;
    float value;
    char *retstr;
{
    if (slider->type == LUI_SLIDER_FLOAT) {
	char tmp[64], *c, *d, *strrchr();

        /* changed by BP on 10-15-91 to a format based upon value */
        if (value>=0.01 || value<=-0.01)
           sprintf(tmp, "%7.2f", value);
	else
	   sprintf(tmp, "%7.5f", value);
	for (c = tmp; *c == ' '; ++c);

        /* remove trailing 0's - WLH 10-21-91 */
          while((d = strrchr(c, '0')) == (c + strlen(c) - 1) &&
                 *(c + strlen(c) - 2) != '.') *d = 0;
	sprintf(retstr, "%s", c);
    }
    else
      sprintf(retstr, "%d", (int)(value+0.5));

    return(strlen(retstr));
}



/******************************************************************************/
/** Added on 5-15-92 by BEP.    To change slider attributes.                  */
LUI_SliderChange( slider, label, min, max, val )
LUI_SLIDER *slider;
char *label;
float min, max, val;
{
   if (slider) {
      slider->label = (char *) malloc( strlen(label)+1 );
      strcpy( slider->label, label );
      slider->low = min;
      slider->high = max;
      slider->value = val;
   }
}



/******************************************************************************/
/********** LUI DESTROY LOGIC * BILLH 10-4-91 *********************************/
/******************************************************************************/

/******************************************************************************/
/* added by billh 10-4-91 */
LUI_SliderBoxDestroy(bp)
LUI_SLIDER_BOX *bp;
{
  LUI_SLIDER_BOX *lbp, *nbp;
  int n, m;

  LUI_SliderFindDestroy(bp);
  lbp = NULL;
  nbp = LUI_SliderBoxHead;
  while (nbp != NULL) {
    if (nbp == bp) {
      /* this is the slider box */
      /* delete from slider box list */
      if (lbp == NULL) LUI_SliderBoxHead = bp->np;
      else lbp->np = bp->np;
      /* delete from event list */
      delete_event(bp->window);
      /* free memory */
      free(bp);
      return;
    }
    lbp = nbp;
    nbp = nbp->np;
  }
  /* not found */
  return;
}


/******************************************************************************/
/* added by billh 10-4-91 */
LUI_SliderFindDestroy(owner)
LUI_SLIDER_BOX *owner;
{
  LUI_SLIDER *bp, *lbp, *nbp;

GOT_ONE:
  bp = LUI_SliderHead;
  while (bp != NULL) {
    if (bp->slider_box == owner) {
      LUI_SliderDestroy(bp);
      goto GOT_ONE;
    }
    bp = bp->np;
  }
  /* not found */
  return;
}


/******************************************************************************/
/* added by billh 10-4-91 */
LUI_SliderDestroy(bp)
LUI_SLIDER *bp;
{
  LUI_SLIDER *lbp, *nbp;
  int n, m;

  lbp = NULL;
  nbp = LUI_SliderHead;
  while (nbp != NULL) {
    if (nbp == bp) {
      /* this is the slider */
      /* delete from slider list */
      if (lbp == NULL) LUI_SliderHead = bp->np;
      else lbp->np = bp->np;
      /* delete from event list */
      delete_event(bp->window);
      /* free memory */
      free(bp);
      return;
    }
    lbp = nbp;
    nbp = nbp->np;
  }
  /* not found */
  return;
}



